// ======================================================================
//
// dllmain.cpp
// jeff grills
//
// copyright 1996 origin systems inc
//
// ======================================================================

#include "first.h"
#include "dllmain.h"

#include "ConfigEngine.h"
#include "ConfigPlatform.h"
#include "Gl_abstract.h"
#include "Gl_init.h"
#include "Material.h"
#include "Palette.h"

// ======================================================================

static BOOL loadDll(void);
static BOOL unloadDll(void);

// ======================================================================

static loaded;

// fatal.h
void (*Fatal)(const char *fmt, ...);
void (*Warning)(const char *fmt, ...);
void (*Message)(const char *fmt, ...);

// Camera.h
Camera::RenderStats *Camera::renderStats;

// ConfigEngine.h
ConfigEngine::Data       *ConfigEngine::data;

// ConfigPlatform.h
ConfigPlatform::Data     *ConfigPlatform::data;

// profile.h
#if USE_PROFILE
int                    *Profile::indx;
Profile::ProfileEntry  *Profile::entry;
const char * const      Profile::startError = "Profile::start overflow DLL";
const char * const      Profile::stopError  = "Profile::stop overflow DLL";
const char * const      Profile::contError  = "Profile::cont overflow DLL";
#endif

// dprint.h
#if USE_DEBUG_PRINT
void (*DoNotUseDPrintClear)(void);
void (*DoNotUseDPrint)(const char *fmt, ...);
bool  *DoNotUseDPrintFlag;
bool  *DoNotUseDPrintPaused;
bool  *DoNotUseDPrintLogging;
bool  *DoNotUseDPrintExplicitLogging;
#endif

// dflag.h
#if USE_DEBUG_FLAG
bool  *DoNotUseDFlag;
#endif

// dkey.h
#if USE_DEBUG_KEY
int   *DoNotUseDKeyContext;
int16 *DoNotUseDKeyPressed;
int16 *DoNotUseDKeyDown;
#endif

// ======================================================================

BOOL WINAPI DllMain(HANDLE /* hInst */, ULONG ul_reason_for_call, LPVOID /* lpReserved */)
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			OutputDebugString("dllmain process attach\n");
			return loadDll();

		case DLL_PROCESS_DETACH:
			OutputDebugString("dllmain process detach\n");
			return unloadDll();

		case DLL_THREAD_ATTACH:
			OutputDebugString("dllmain thread attach\n");
			return TRUE;

		case DLL_THREAD_DETACH:
			OutputDebugString("dllmain thread detach\n");
			return TRUE;
	}

	return FALSE;
}

// ----------------------------------------------------------------------

void DestroyGl(GL_abstract *gl)
{
	delete gl;
}

// ----------------------------------------------------------------------

BOOL loadDll(void)
{
	if (!loaded)
	{
		// first load
		OutputDebugString("dll first load\n");
	}

	loaded++;
	return TRUE;
}

// ----------------------------------------------------------------------

BOOL unloadDll(void)
{
	if (--loaded == 0)
		// last exit for the lost
		OutputDebugString("dll last unload\n");

	return TRUE;
}

// ----------------------------------------------------------------------

void InitGl(const GL_init *init)
{
	DLLMemoryManager::alloc       = init->MemoryManager__alloc;
	DLLMemoryManager::free        = init->MemoryManager__free;

	MemoryManager::getGuardPattern = init->MemoryManager__getGuardPattern;
	MemoryManager::getUserPattern  = init->MemoryManager__getUserPattern;
	MemoryManager::getFreePattern  = init->MemoryManager__getFreePattern;

	Fatal                         = init->Fatal;
	Warning                       = init->Warning;
	Message                       = init->Message;

	Camera::renderStats           = init->cameraRenderStats;

	ConfigEngine::data            = init->engineParmData;
	ConfigPlatform::data          = init->platformParmData;

	PROFILE_SET_DLL(init->Profile__indx, init->Profile__entry);

#if USE_DEBUG_PRINT
	DoNotUseDPrintClear           = init->doNotUseDPrintClear;
	DoNotUseDPrint                = init->doNotUseDPrint;
	DoNotUseDPrintFlag            = init->doNotUseDPrintFlag;
	DoNotUseDPrintPaused          = init->doNotUseDPrintPaused;
	DoNotUseDPrintLogging         = init->doNotUseDPrintLogging;
	DoNotUseDPrintExplicitLogging = init->doNotUseDPrintExplicitLogging;
#endif

#if USE_DEBUG_FLAG
	DoNotUseDFlag                 = init->doNotUseDFlag;
#endif

#if USE_DEBUG_KEY
	DoNotUseDKeyContext           = init->doNotUseDKeyContext;
	DoNotUseDKeyPressed           = init->doNotUseDKeyPressed;
	DoNotUseDKeyDown              = init->doNotUseDKeyDown;
#endif
}

// ======================================================================
